@@ -31,17 +31,19 @@
 from __future__ import division
 
 name    = "datavision"
-version = "2016-02-11T1235Z"
+version = "2016-04-14T1500Z"
 
-import os
-import sys
-import math
-import random
 import itertools
-import operator
+import math
 import matplotlib.pyplot
 import numpy
+import operator
+import os
+import PIL.Image
+import random
 import scipy.stats
+import sys
+
 import pyprel
 import shijian
 
@@ -343,6 +345,130 @@ def plot_list(
                 dpi = 700
             )
 
+def generate_sine_values(
+    frequency   = 5,
+    sample_rate = 16000,
+    time        = 10
+    ):
+    sampling_interval = time / sample_rate
+    values_time       = numpy.arange(0, time, sampling_interval)
+    values_amplitude  = numpy.sin(2 * numpy.pi * frequency * values_time)
+    return values_amplitude, values_time
+
+def generate_composite_sine_values(
+    frequencies = [5, 10, 15],
+    sample_rate = 16000,
+    time        = 10
+    ):
+    values_amplitude_list = []
+    for frequency in frequencies:
+        values_amplitude, values_time = generate_sine_values(
+            frequency   = frequency,
+            time        = time,
+            sample_rate = sample_rate
+        )
+        values_amplitude_list.append(values_amplitude)
+    values_amplitude_sum = sum(values_amplitude_list)
+    return values_amplitude_sum, values_time
+
+def greatest_frequency_contributions_FFT(
+    values_amplitude        = None,
+    values_time             = None,
+    time                    = None,
+    sample_rate             = 16000,
+    number_of_contributions = 10
+    ):
+
+    if values_time is None:
+        sampling_interval = time / sample_rate
+        values_time       = numpy.arange(0, time, sampling_interval)
+    signal_length     = len(values_amplitude)
+    values_indices    = numpy.arange(signal_length)
+    if time is None:
+        time = signal_length / sample_rate
+    values_time = numpy.linspace(0, time, num = signal_length)
+    # two sides frequency range
+    frequencies       = values_indices / time
+    # one side frequency range
+    frequencies       = frequencies[range(int(signal_length / 2))] 
+    # FFT
+    weightings        = numpy.fft.fft(values_amplitude) / signal_length
+    # normalization
+    weightings        = weightings[range(int(signal_length / 2))]
+
+    weightings_absolute = [abs(weighting) for weighting in list(weightings)]
+
+    indices = shijian.indices_of_greatest_values(
+        weightings_absolute,
+        number = number_of_contributions
+    )
+    frequencies = [frequencies[index] for index in indices]
+
+    return frequencies
+
+def save_FFT_plot_matplotlib(
+    values_amplitude       = None,
+    values_time            = None,
+    time                   = None,
+    sample_rate            = 16000,
+    filename               = "FFT.png",
+    title_axis_x_amplitude = "time",
+    title_axis_y_amplitude = "amplitude",
+    title_axis_x_FFT       = "frequency",
+    title_axis_y_FFT       = "|weighting|"
+    ):
+
+    if values_time is None:
+        sampling_interval = time / sample_rate
+        values_time       = numpy.arange(0, time, sampling_interval)
+    signal_length     = len(values_amplitude)
+    values_indices    = numpy.arange(signal_length)
+    if time is None:
+        time = signal_length / sample_rate
+    values_time = numpy.linspace(0, time, num = signal_length)
+    # two sides frequency range
+    frequencies       = values_indices / time
+    # one side frequency range
+    frequencies       = frequencies[range(int(signal_length / 2))] 
+    # FFT
+    weightings        = numpy.fft.fft(values_amplitude) / signal_length
+    # normalization
+    weightings        = weightings[range(int(signal_length / 2))]
+
+    figure, axes      = matplotlib.pyplot.subplots(2, 1)
+    # plot amplitude versus time
+    axes[0].plot(values_time, values_amplitude, "b", linewidth = 0.5)
+    axes[0].set_xlabel(title_axis_x_amplitude)
+    axes[0].set_ylabel(title_axis_y_amplitude)
+    # plot FFT
+    axes[1].plot(frequencies, abs(weightings), "r", linewidth = 0.5)
+    axes[1].set_xlabel(title_axis_x_FFT)
+    axes[1].set_ylabel(title_axis_y_FFT)
+
+    figure.savefig(
+        filename,
+        dpi = 700
+    )
+
+def dot_product(v1, v2):
+    return(sum((a * b) for a, b in zip(v1, v2)))
+
+def magnitude(v):
+    return(numpy.linalg.norm(v))
+    #return(math.sqrt(dot_product(v, v)))
+
+def angle(v1, v2):
+    cosine = dot_product(v1, v2) / (magnitude(v1) * magnitude(v2))
+    cosine = 1 if cosine > 1 else cosine
+    return(math.acos(cosine))
+
+def composite_variable(x):
+    k = len(x) + 1
+    variable = 0
+    for index, element in enumerate(x):
+        variable += k**(index - 1) * element
+    return variable
+
 def normalize(
     x,
     summation = None
@@ -2128,3 +2254,17 @@ def reverse_y(matrix):
     reverse y-axis of 2D matrix
     """
     return [list(reversed(matrix_i)) for matrix_i in matrix]
+
+def difference_RMS_images(
+    filename_1 = None,
+    filename_2 = None
+    ):
+    histogram_1 = PIL.Image.open(filename_1).histogram()
+    histogram_2 = PIL.Image.open(filename_2).histogram()
+    try:
+        RMS = math.sqrt(reduce(operator.add, map(
+            lambda a, b: (a - b)**2, histogram_1, histogram_2)) / len(histogram_1)
+        )
+    except:
+        RMS = None
+    return RMS
